@@ -283,7 +283,7 @@ extern void pixmap_cache_set (Lisp_Image *im, int width, int height,
 			      Pixmap p1, Pixmap p2);
 extern void pixmap_cache_flush_image (Lisp_Image *im);
 #endif
-extern repv Fpixmap_cache_control (repv max);
+extern repv Fpixmap_cache_control (repv max, repv reset);
 extern void pixmap_cache_init (void);
 
 /* from property-cache.c */
@@ -311,7 +311,6 @@ extern int window_type;
 extern Lisp_Window *focus_window;
 extern int pending_destroys;
 extern repv Qadd_window_hook, Qbefore_add_window_hook, Qplace_window_hook;
-extern bool mapped_not_override_p (Window id);
 extern void commit_queued_focus_change (void);
 extern void focus_on_window (Lisp_Window *w);
 extern void focus_off_window (Lisp_Window *w);
